//-----------------------------------------------------------------------------
//
// ClassName:    IAxis
//  
// Description:  Definition of the IAxis interface class
//
//               IAxis represents a binned histogram axis.
//               A 1D histogram has one axis representing the X axis,
//               a 2D histogram has two axes representing the X and Y Axis. 
//
//  The AIDA group.
//
// 04/02/2000 : Pavel Binko, Dino Ferrero Merlino, Andreas Pfeiffer :
//   First version.
//-----------------------------------------------------------------------------

#ifndef IAxis_h
#define IAxis_h

class IAxis {

public:
  virtual ~IAxis() { /* nop */ }

public:

  /// =========================================================================
  /// Axis edges
  /// =========================================================================

  /// Lower axis edge
  virtual double lowerEdge() const = 0;

  /// Upper axis edge
  virtual double upperEdge() const = 0;


  /// =========================================================================
  /// Bins
  /// =========================================================================

  /// -------------------------------------------------------------------------
  /// Number of bins

  /// Number of in-range bins in the axis
  virtual int bins() const = 0;

  /// -------------------------------------------------------------------------
  /// Bin edges

  /// Lower edge of the in-range bin identified by index
  virtual double binLowerEdge( int index ) const = 0;

  /// Upper edge of the in-range bin identified by index
  virtual double binUpperEdge( int index ) const = 0;

  /// -------------------------------------------------------------------------
  /// Bin widths

  /// Width of the in-range bin identified by index
  virtual double binWidth( int index ) const = 0;

  /// -------------------------------------------------------------------------
  /// Bin centres

  /// Centre of the bin located by index
  virtual double binCentre( int index ) const = 0;

  /// =========================================================================
  /// Conversions between coordinates and bin indices
  /// =========================================================================

  /// Equivalent to coordToIndex( coord ); for 1D histogram
  ///   or coordToIndexX( coordX ); or coordToIndexY( coordY ); for 2D histogram
  virtual int coordToIndex( double coord ) const = 0;

};


#endif

