#ifndef AIDA_IHISTOGRAMFACTORY_H
#define AIDA_IHISTOGRAMFACTORY_H

#include <string>
#include <vector>

#include "IHistogram1D.h"
#include "IHistogram2D.h"
#include "IHistogram3D.h"

/**
 * @author Mark Donszelmann
 * @version $Id: IHistogramFactory.h,v 1.1 2001/05/01 19:01:06 duns Exp $
 */

class IHistogramFactory {

    public:
        virtual ~IHistogramFactory() { };
        
        virtual IHistogram1D* create1D(std::string nameAndLabel, 
                               int nBins, double lowerEdge, double upperEdge) = 0;                         
        virtual IHistogram1D* create1D(std::string name, std::string label, 
                               int nBins, double lowerEdge, double upperEdge) = 0;
        virtual IHistogram1D* create1D(std::string name, std::string label,
                               int nBins, double lowerEdge, double upperEdge,
                               std::string options) = 0;
        virtual IHistogram1D* create1D(std::string name, std::string label,
                               std::vector<double> binEdge, std::string options) = 0;
                               
        virtual IHistogram2D* create2D(std::string nameAndLabel,
                               int nBinsX, double lowerEdgeX, double upperEdgeX,
                               int nBinsY, double lowerEdgeY, double upperEdgeY) = 0;
        virtual IHistogram2D* create2D(std::string name, std::string label, 
                               int nBinsX, double lowerEdgeX, double upperEdgeX,
                               int nBinsY, double lowerEdgeY, double upperEdgeY) = 0;
        virtual IHistogram2D* create2D(std::string name, std::string label, 
                               int nBinsX, double lowerEdgeX, double upperEdgeX,
                               int nBinsY, double lowerEdgeY, double upperEdgeY,
                               std::string options) = 0;
        virtual IHistogram2D* create2D(std::string name, std::string label, 
                               std::vector<double> binEdgeX, 
                               std::vector<double> binEdgeY,
                               std::string options) = 0;
                               
        virtual IHistogram3D* create3D(std::string nameAndLabel,
                               int nBinsX, double lowerEdgeX, double upperEdgeX,
                               int nBinsY, double lowerEdgeY, double upperEdgeY,
                               int nBinsZ, double lowerEdgeZ, double upperEdgeZ) = 0;
        virtual IHistogram3D* create3D(std::string name, std::string label, 
                               int nBinsX, double lowerEdgeX, double upperEdgeX,
                               int nBinsY, double lowerEdgeY, double upperEdgeY,
                               int nBinsZ, double lowerEdgeZ, double upperEdgeZ) = 0;
        virtual IHistogram3D* create3D(std::string name, std::string label, 
                               int nBinsX, double lowerEdgeX, double upperEdgeX,
                               int nBinsY, double lowerEdgeY, double upperEdgeY,
                               int nBinsZ, double lowerEdgeZ, double upperEdgeZ,
                               std::string options) = 0;
        virtual IHistogram3D* create3D(std::string name, std::string label, 
                               std::vector<double> binEdgeX, 
                               std::vector<double> binEdgeY,
                               std::vector<double> binEdgeZ,
                               std::string options) = 0;
};

#endif
