
#ifndef AIDA_IHISTOGRAM_H 
#define AIDA_IHISTOGRAM_H 1

/**
 * User level interface to Histogram.
 *
 * - NOTE: fillable or non-fillable
 *
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version 2.0, 06-02-2001
 */

#include <string>

#include "AIDA/IAIDAConstants.h"

class IHistogram : virtual public IAIDAConstants {

	/**
	 * Label of the histogram
	 *
	 * @returns null if label was not set.
	 */ 
	virtual const std::string & label();
	
	virtual void setLabel(const std::string & label);
	/**
	 * Title of the histogram (will be set only in the constructor).
	 * @deprecated please use label()
	 */ 
	virtual const std::string & title();    
	/**
	 * to be defined
	 */
    virtual IAnnotation annotation();
	/**
	 * Returns 1 for one-dimensional histograms, 2 for two-dimensional histograms, and so on.
	 */ 
	virtual int dimension();    
	/**
	 * Reset contents; as if just constructed.
	 */
	virtual void reset();    
	/**
	 * Number of in-range entries in the histogram.
	 */ 
	virtual int entries();    
	/**
	 * Number of all entries in all (both in-range and under/overflow) bins in the histogram.
	 */ 
	virtual int allEntries();    
	/**
	 * Number of under and overflow entries in the histogram.
	 */ 
	virtual int extraEntries();    
	/** 
	 * Number of equivalent entries.
	 * @return <tt>SUM[ weight ] ^ 2 / SUM[ weight^2 ]</tt>.
	 */ 
	virtual double equivalentBinEntries();    
	/**
	 * Sum of in-range bin heights in the histogram.
	 */ 
	virtual double sumBinHeights();    
    /**
     * Sum of all (both in-range and under/overflow) bin heights in the histogram.
     */ 
    virtual double sumAllBinHeights();    
	/**
	 * Sum of under/overflow bin heights in the histogram.
	 */ 
	virtual double sumExtraBinHeights();    
	/**
	 * Minimum height of in-range bins in the histogram.
	 */ 
	virtual double minBinHeight();    
	/**
	 * Maximum height of in-range bins in the histogram.
	 */ 
	virtual double maxBinHeight();    


    /**
     * Shows the histo.
     */
    virtual void show();
    
};

#endif 
