
// This file has been automatically converted from IHistogram1D.java 

#ifndef AIDA_IHISTOGRAM1D_H 
#define AIDA_IHISTOGRAM1D_H 1

/**
 * User level interface to 1D Histogram.
 * 
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version 2.0, 06-02-2001
 */

// type ahead declarations 
class IAxis;
class IHistogram1D;
class IFitFunction;

// base class(es)  
#include "AIDA/IHistogram.h"


class IHistogram1D  :  virtual public IHistogram {

public:

   virtual ~IHistogram1D () { /* nop */; }
	/**
	 * Fill histogram with weight 1.
	 */ 
   virtual void fill ( double x ) = 0;
	/**
	 * Fill histogram with specified weight.
	 */
   virtual void fill ( double x, double weight ) = 0;
	/**
	 * Number of entries in the corresponding bin (ie the number of times fill was called for this bin).
	 * @param index the bin number (0...N-1) or OVERFLOW or UNDERFLOW.
	 */
   virtual int binEntries ( int index ) = 0;
	/**
	 * Total height of the corresponding bin (ie the sum of the weights in this bin).
	 * @param index the bin number (0...N-1) or OVERFLOW or UNDERFLOW.
	 */ 
   virtual double binHeight ( int index ) = 0;
	/**
	 * The error on this bin.
	 * @param index the bin number (0...N-1) or OVERFLOW or UNDERFLOW.
	 */ 
   virtual double binError ( int index ) = 0;
	/**
	 * Returns the mean of the whole histogram as calculated on filling-time.
	 */ 
   virtual double mean (  ) = 0;
	/**
	 * Returns the rms of the whole histogram as calculated on filling-time.
	 */ 
   virtual double rms (  ) = 0;
	/**
	 * Returns the X axis.
	 */
   virtual IAxis * axis (  ) = 0;
	/**
	 * Convenience method, equivalent to <tt>axis().coordToIndex(coord)</tt>.
	 * @see IAxis#coordToIndex(double)
	 */
   virtual int coordToIndex ( double coord ) = 0;

    /**
     * Adds two histograms
     *
     * @returns histo + h
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram1D * add ( const IHistogram1D & h ) = 0;
    
    /**
     * Subtracts two histograms
     *
     * @returns histo - h
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram1D * sub ( const IHistogram1D & h ) = 0;
    
    /**
     * Multiplies two histograms
     *
     * @returns histo * h
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram1D * mul ( const IHistogram1D & h ) = 0;
    
    /**
     * Divides two histograms
     *
     * @returns histo / h
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram1D * div ( const IHistogram1D & h ) = 0;
    
    /**
     * Fit the given function to the histogram. Subsequent plotting
     * of this histogram will also show the fit-function.
     * Any previously fitted function will be removed.
     *
     * @see #clearFit()
     * @returns chi-square
     */
   virtual double fit ( const IFitFunction & fitFunction ) = 0;
    
    
    /**
     * Removes the fit function from the histogram.
     */
   virtual void clearFit (  ) = 0;
};



#endif /* ifndef AIDA_IHISTOGRAM1D_H */ 

