
// This file has been automatically converted from IHistogram2D.java 

#ifndef AIDA_IHISTOGRAM2D_H 
#define AIDA_IHISTOGRAM2D_H 1

/**
 * User level interface to 2D Histogram.
 * 
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version 2.0, 06-02-2001
 */

// type ahead declarations 
class IAxis;
class IHistogram1D;
class IHistogram2D;

// base class(es)  
#include "AIDA/IHistogram.h"


class IHistogram2D  :  virtual public IHistogram {

public:

   virtual ~IHistogram2D () { /* nop */; }
	/**
	 * Fill the histogram with weight 1.
	 */
   virtual void fill ( double x, double y ) = 0;
	/**
	 * Fill the histogram with specified weight.
	 */
   virtual void fill ( double x, double y, double weight ) = 0;
	/**
	 * The number of entries (ie the number of times fill was called for this bin).
	 * @param indexX the x bin number (0...Nx-1) or OVERFLOW or UNDERFLOW.
	 * @param indexY the y bin number (0...Ny-1) or OVERFLOW or UNDERFLOW.
	 */
   virtual int binEntries ( int indexX, int indexY ) = 0;
	/**
	 * Equivalent to <tt>projectionX().binEntries(indexX)</tt>.
	 */
   virtual int binEntriesX ( int indexX ) = 0;
	/**
	 * Equivalent to <tt>projectionY().binEntries(indexY)</tt>.
	 */
   virtual int binEntriesY ( int indexY ) = 0;
	/**
	 * Total height of the corresponding bin (ie the sum of the weights in this bin).
	 * @param indexX the x bin number (0...Nx-1) or OVERFLOW or UNDERFLOW.
	 * @param indexY the y bin number (0...Ny-1) or OVERFLOW or UNDERFLOW.
	 */ 
   virtual double binHeight ( int indexX, int indexY ) = 0;
	/**
	 * Equivalent to <tt>projectionX().binHeight(indexX)</tt>.
	 */
   virtual double binHeightX ( int indexX ) = 0;
	/**
	 * Equivalent to <tt>projectionY().binHeight(indexY)</tt>.
	 */
   virtual double binHeightY ( int indexY ) = 0;
	/**
	 * The error on this bin.
	 * @param indexX the x bin number (0...Nx-1) or OVERFLOW or UNDERFLOW.
	 * @param indexY the y bin number (0...Ny-1) or OVERFLOW or UNDERFLOW.
	 */ 
   virtual double binError ( int indexX, int indexY ) = 0;
	/**
	 *  Returns the mean of the histogram, as calculated on filling-time projected on the X axis.
	 */ 
   virtual double meanX (  ) = 0;
	/**
	 *  Returns the mean of the histogram, as calculated on filling-time projected on the Y axis.
	 */ 
   virtual double meanY (  ) = 0;
	/**
	 * Returns the rms of the histogram as calculated on filling-time projected on the X axis.
	 */
   virtual double rmsX (  ) = 0;
	/**
	 * Returns the rms of the histogram as calculated on filling-time projected on the Y axis.
	 */
   virtual double rmsY (  ) = 0;
	/**
	 * Return the X axis.
	 */
   virtual IAxis * xAxis (  ) = 0;
	/**
	 * Return the Y axis.
	 */
   virtual IAxis * yAxis (  ) = 0;
	/**
	 * Convenience method, equivalent to <tt>xAxis().coordToIndex(coord)</tt>.
	 * @see IAxis#coordToIndex(double)
	 */  
   virtual int coordToIndexX ( double coordX ) = 0;
	/**
	 * Convenience method, equivalent to <tt>yAxis().coordToIndex(coord)</tt>.
	 * @see IAxis#coordToIndex(double)
	 */  
   virtual int coordToIndexY ( double coordY ) = 0;

	/**
	 * Create a projection parallel to the X axis.
	 * Equivalent to <tt>sliceX(UNDERFLOW_BIN,OVERFLOW_BIN)</tt>.
	 */
   virtual IHistogram1D * projectionX (  ) = 0;
	/**
	 * Create a projection parallel to the Y axis.
	 * Equivalent to <tt>sliceY(UNDERFLOW_BIN,OVERFLOW_BIN)</tt>.
	 */
   virtual IHistogram1D * projectionY (  ) = 0;
	/**
	 * Slice parallel to the Y axis at bin indexY and one bin wide.
	 * Equivalent to <tt>sliceX(indexY,indexY)</tt>.
	 */
   virtual IHistogram1D * sliceX ( int indexY ) = 0;
	/**
	 * Slice parallel to the X axis at bin indexX and one bin wide.
	 * Equivalent to <tt>sliceY(indexX,indexX)</tt>.
	 */
   virtual IHistogram1D * sliceY ( int indexX ) = 0;
	/**
	 * Create a slice parallel to the X axis, 
	 * between "indexY1" and "indexY2" (inclusive).
	 * The returned IHistogram1D represents an instantaneous snapshot of the
	 * histogram at the time the slice was created.
	 */ 
   virtual IHistogram1D * sliceX ( int indexY1, int indexY2 ) = 0;
	/**
	 * Create a slice parallel to the Y axis, 
	 * between "indexX1" and "indexX2" (inclusive).
	 * The returned IHistogram1D represents an instantaneous snapshot of the
	 * histogram at the time the slice was created.
	 */ 
   virtual IHistogram1D * sliceY ( int indexX1, int indexX2 ) = 0;
        
        /**
     * Adds two histograms
     *
     * @returns histo + h
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram2D * add ( const IHistogram2D & h ) = 0;
    
    /**
     * Subtracts two histograms
     *
     * @returns histo - h
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram2D * sub ( const IHistogram2D & h ) = 0;
    
    /**
     * Multiplies two histograms
     *
     * @returns histo * h
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram2D * mul ( const IHistogram2D & h ) = 0;
    
    /**
     * Divides two histograms
     *
     * @returns histo / h
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram2D * div ( const IHistogram2D & h ) = 0;
    
};
#endif /* ifndef AIDA_IHISTOGRAM2D_H */ 

