
// This file has been automatically converted from IHistogram3D.java 

#ifndef AIDA_IHISTOGRAM3D_H 
#define AIDA_IHISTOGRAM3D_H 1

/**
 * User level interface to 3D Histogram.
 *
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version 2.0, 06-02-2001
 */

// type ahead declarations 
class IAxis;
class IHistogram3D;
class IHistogram2D;

// base class(es)  
#include "AIDA/IHistogram.h"


class IHistogram3D  :  virtual public IHistogram {

public:

   virtual ~IHistogram3D () { /* nop */; }
        /**
         * Fill the histogram with weight 1.
         */
   virtual void fill ( double x, double y, double z ) = 0;
        /**
         * Fill the histogram with specified weight.
         */
   virtual void fill ( double x, double y, double z, double weight ) = 0;
        /**
         * The number of entries (ie the number of times fill was called for this bin).
         * @param indexX the x bin number (0...Nx-1) or OVERFLOW or UNDERFLOW.
         * @param indexY the y bin number (0...Ny-1) or OVERFLOW or UNDERFLOW.
         */
   virtual int binEntries ( int indexX, int indexY, int indexZ ) = 0;
        /**
         * Equivalent to <tt>projectionXY().binEntriesX(indexX)</tt>.
         */
   virtual int binEntriesX ( int indexX ) = 0;
        /**
         * Equivalent to <tt>projectionXY().binEntriesY(indexY)</tt>.
         */
   virtual int binEntriesY ( int indexY ) = 0;
        /**
         * Equivalent to <tt>projectionXZ().binEntriesY(indexZ)</tt>.
         */
   virtual int binEntriesZ ( int indexZ ) = 0;
        /**
         * Total height of the corresponding bin (ie the sum of the weights in this bin).
         * @param indexX the x bin number (0...Nx-1) or OVERFLOW or UNDERFLOW.
         * @param indexY the y bin number (0...Ny-1) or OVERFLOW or UNDERFLOW.
         */
   virtual double binHeight ( int indexX, int indexY, int indexZ ) = 0;
        /**
         * Equivalent to <tt>projectionXY().binHeightX(indexX)</tt>.
         */
   virtual double binHeightX ( int indexX ) = 0;
        /**
         * Equivalent to <tt>projectionXY().binHeightY(indexY)</tt>.
         */
   virtual double binHeightY ( int indexY ) = 0;
        /**
         * Equivalent to <tt>projectionXZ().binHeightY(indexZ)</tt>.
         */
   virtual double binHeightZ ( int indexZ ) = 0;
        /**
         * The error on this bin.
         * @param indexX the x bin number (0...Nx-1) or OVERFLOW or UNDERFLOW.
         * @param indexY the y bin number (0...Ny-1) or OVERFLOW or UNDERFLOW.
         */
   virtual double binError ( int indexX, int indexY, int indexZ ) = 0;
        /**
         *  Returns the mean of the histogram, as calculated on filling-time projected on the X axis.
         */
   virtual double meanX (  ) = 0;
        /**
         *  Returns the mean of the histogram, as calculated on filling-time projected on the Y axis.
         */
   virtual double meanY (  ) = 0;
        /**
         *  Returns the mean of the histogram, as calculated on filling-time projected on the Z axis.
         */
   virtual double meanZ (  ) = 0;
        /**
         * Returns the rms of the histogram as calculated on filling-time projected on the X axis.
         */
   virtual double rmsX (  ) = 0;
        /**
         * Returns the rms of the histogram as calculated on filling-time projected on the Y axis.
         */
   virtual double rmsY (  ) = 0;
        /**
         * Returns the rms of the histogram as calculated on filling-time projected on the Z axis.
         */
   virtual double rmsZ (  ) = 0;
        /**
         * Return the X axis.
         */
   virtual IAxis * xAxis (  ) = 0;
        /**
         * Return the Y axis.
         */
   virtual IAxis * yAxis (  ) = 0;
        /**
         * Return the Z axis.
         */
   virtual IAxis * zAxis (  ) = 0;
        /**
         * Convenience method, equivalent to <tt>xAxis().coordToIndex(coord)</tt>.
         * @see IAxis#coordToIndex(double)
         */
   virtual int coordToIndexX ( double coordX ) = 0;
        /**
         * Convenience method, equivalent to <tt>yAxis().coordToIndex(coord)</tt>.
         * @see IAxis#coordToIndex(double)
         */
   virtual int coordToIndexY ( double coordY ) = 0;
        /**
         * Convenience method, equivalent to <tt>zAxis().coordToIndex(coord)</tt>.
         * @see IAxis#coordToIndex(double)
         */
   virtual int coordToIndexZ ( double coordZ ) = 0;
        /**
         * Create a projection parallel to the X axis.
         * Equivalent to <tt>sliceXY(UNDERFLOW_BIN,OVERFLOW_BIN)</tt>.
         */
   virtual IHistogram2D * projectionXY (  ) = 0;
        /**
         * Create a projection parallel to the Z axis.
         * Equivalent to <tt>sliceXZ(UNDERFLOW_BIN,OVERFLOW_BIN)</tt>.
         */
   virtual IHistogram2D * projectionXZ (  ) = 0;
        /**
         * Create a projection parallel to the Y axis.
         * Equivalent to <tt>sliceYZ(UNDERFLOW_BIN,OVERFLOW_BIN)</tt>.
         */
   virtual IHistogram2D * projectionYZ (  ) = 0;
        /**
         * Create a slice perpendicular to the Z axis,
         * between "indexZ1" and "indexZ2" (inclusive).
         * The returned IHistogram2D represents an instantaneous snapshot of the
         * histogram at the time the slice was created.
         * The X axis of the returned histogram corresponds to the X axis of this histogram.
         * The Y axis of the returned histogram corresponds to the Y axis of this histogram.
         */
   virtual IHistogram2D * sliceXY ( int indexZ1, int indexZ2 ) = 0;
        /**
         * Create a slice perpendicular to the Y axis,
         * between "indexY1" and "indexY2" (inclusive).
         * The returned IHistogram2D represents an instantaneous snapshot of the
         * histogram at the time the slice was created.
         * The X axis of the returned histogram corresponds to the X axis of this histogram.
         * The Y axis of the returned histogram corresponds to the Z axis of this histogram.
         */
   virtual IHistogram2D * sliceXZ ( int indexY1, int indexY2 ) = 0;
        /**
         * Create a slice perpendicular to the X axis,
         * between "indexX1" and "indexX2" (inclusive).
         * The returned IHistogram2D represents an instantaneous snapshot of the
         * histogram at the time the slice was created.
         * The X axis of the returned histogram corresponds to the Y axis of this histogram.
         * The Y axis of the returned histogram corresponds to the Z axis of this histogram.
         */
   virtual IHistogram2D * sliceYZ ( int indexX1, int indexX2 ) = 0;
    
           /**
            * Adds two histograms
            *
            * @returns histo + h
            * @throws IllegalArgumentException if histogram binnings are incompatible
            */
   virtual IHistogram3D * add ( const IHistogram3D & h ) = 0;
    
    /**
     * Subtracts two histogram     *
     * @returns histo - h
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram3D * sub ( const IHistogram3D & h ) = 0;
    
    /**
     * Multiplies two histograms
     *
     * @returns histo * h
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram3D * mul ( const IHistogram3D & h ) = 0;
    
    /**
     * Divides two histograms
     *
     * @returns histo / h
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram3D * div ( const IHistogram3D & h ) = 0;
};


#endif /* ifndef AIDA_IHISTOGRAM3D_H */ 

