
// This file has been automatically converted from ITree.java 

#ifndef AIDA_ITREE_H 
#define AIDA_ITREE_H 1


/**
 * User level interface to a Tree.
 *
 * All paths follow unix convention .., ., /
 * Backslash is the escape character
 * Relative paths are allowed
 *
 * NOTE: - this tree keeps a current position (pwd) within the tree.
 *
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version 2.0, 06-02-2001
 */

// includes 
#include <string>

// type ahead declarations 
class IManagedObject;
class Iterator;
class ITree;


class ITree  {

public:

   virtual ~ITree () { /* nop */; }
    
   virtual IManagedObject * find ( const std::string & path ) = 0;
    
   virtual void cd ( const std::string & path ) = 0;
   virtual const std::string & pwd (  ) = 0;
    
    // All list methods return all managed objects,
    // including folders, but not symlinks
   virtual Iterator * ls (  ) = 0;
   virtual Iterator * ls ( bool recursive ) = 0;
   virtual Iterator * ls ( const std::string & path ) = 0;
   virtual Iterator * ls ( const std::string & path, bool recursive ) = 0;
    
    // path = subdir.../dir
    // throws exception if 
    // - subdir does not exist
    // - dir does exist
    // - subdir is not a folder
   virtual void mkdir ( const std::string & path ) = 0;
    
    // path = subdir.../dir
    // throws exception if 
    // - subdir is not a folder
   virtual void mkdirs ( const std::string & path ) = 0;
    
   virtual void rmdir ( const std::string & path ) = 0;
    // removing a mount point should commit, close and delete the tree object    
   virtual void rm ( const std::string & path ) = 0;
   virtual const std::string & findPath ( const IManagedObject & object ) = 0;
   virtual void mv ( const std::string & oldPath, const std::string & newPath ) = 0;

   virtual void commit (  ) = 0;
   virtual void cp ( const std::string & oldPath, const std::string & newPath ) = 0;
   virtual void cp ( const std::string & oldPath, const std::string & newPath, bool recursive ) = 0;
   virtual void symlink ( const std::string & path, const std::string & alias ) = 0;
    // a tree can only be mounted once
   virtual void mount ( const std::string & path, const ITree & tree, const std::string & treePath ) = 0;
   virtual void unmount ( const std::string & path ) = 0;
    // close calls commit before closing
   virtual void close (  ) = 0;
};
#endif /* ifndef AIDA_ITREE_H */ 

