
// This file has been automatically converted from IAnalysisFactory.java 

#ifndef AIDA_IANALYSISFACTORY_H 
#define AIDA_IANALYSISFACTORY_H 1
/**
 * The "master" factory from which other factories are obtained.
 * Typically accessed by: 
 * <pre>
 *   IAnalysisFactory af = (IAnalysisFactory) ServiceManager.create("hep.aida.IAnalysisFactory");
 * </pre>
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version $Id: IAnalysisFactory.h,v 1.1 2001/08/07 11:36:13 pfeiffer Exp $
 */
// C++ IAnalysisFactory* af = AIDA_createIAnalysisFactory();


// type ahead declarations 
class ICloudFactory;
class ITupleFactory;
class ITreeFactory;
class IHistogramFactory;
class ITree;


class IAnalysisFactory  {

public:

   virtual ~IAnalysisFactory () { /* nop */; }
    /**
     * Create an ITreeFactory.
     */
   virtual ITreeFactory * createTreeFactory (  ) = 0;
    /**
     * Create an IHistogramFactory.
     * @param tree The ITree which created histograms will be associated
     */
   virtual IHistogramFactory * createHistogramFactory ( const ITree & tree ) = 0;
    /**
     * Create an ICloudFactory.
     * @param tree The ITree which created clouds will be associated
     */
   virtual ICloudFactory * createCloudFactory ( const ITree & tree ) = 0;
    /**
     * Create an ITupleFactory.
     * @param tree The ITree which created tuples will be associated
     */
   virtual ITupleFactory * createTupleFactory ( const ITree & tree ) = 0;
};
#endif /* ifndef AIDA_IANALYSISFACTORY_H */ 

