
// This file has been automatically converted from IHistogramFactory.java 

#ifndef AIDA_IHISTOGRAMFACTORY_H 
#define AIDA_IHISTOGRAMFACTORY_H 1

/**
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version $Id: IHistogramFactory.h,v 1.4 2001/08/08 09:35:42 pfeiffer Exp $
 */

// includes 
#include <vector>
#include <string>

// type ahead declarations 
class IHistogram2D;
class IHistogram1D;
class IHistogram3D;


class IHistogramFactory  {

public:

   virtual ~IHistogramFactory () { /* nop */; }

   virtual IHistogram1D * create1D ( const std::string & name, const std::string & label, int nBins, double lowerEdge, double upperEdge, const std::string & options = "") = 0;

   virtual IHistogram1D * create1D ( const std::string & name, const std::string & label, const std::vector< double > & binEdge, const std::string & options ) = 0;

   virtual IHistogram1D * create1D ( const std::string & nameAndLabel                   , int nBins, double lowerEdge, double upperEdge ) = 0;
                                                                                          
                                 
                                 
   virtual IHistogram2D * create2D ( const std::string & name, const std::string & label, int nBinsX, double lowerEdgeX, double upperEdgeX, int nBinsY, double lowerEdgeY, double upperEdgeY, const std::string & options  = "") = 0;

   virtual IHistogram2D * create2D ( const std::string & name, const std::string & label, const std::vector< double > & binEdgeX, const std::vector< double > & binEdgeY, const std::string & options ) = 0;

   virtual IHistogram2D * create2D ( const std::string & nameAndLabel                   , int nBinsX, double lowerEdgeX, double upperEdgeX, int nBinsY, double lowerEdgeY, double upperEdgeY ) = 0;
                                
                                 
   virtual IHistogram3D * create3D ( const std::string & name, const std::string & label, int nBinsX, double lowerEdgeX, double upperEdgeX, int nBinsY, double lowerEdgeY, double upperEdgeY, int nBinsZ, double lowerEdgeZ, double upperEdgeZ, const std::string & options = "" ) = 0;
                                
   virtual IHistogram3D * create3D ( const std::string & name, const std::string & label, const std::vector< double > & binEdgeX, const std::vector< double > & binEdgeY, const std::vector< double > & binEdgeZ, const std::string & options ) = 0;
                                
   virtual IHistogram3D * create3D ( const std::string & nameAndLabel                   , int nBinsX, double lowerEdgeX, double upperEdgeX, int nBinsY, double lowerEdgeY, double upperEdgeY, int nBinsZ, double lowerEdgeZ, double upperEdgeZ ) = 0;

    // Methods which create histograms by performing operations on existing histograms
    // 1D Histograms 
    /**
     * Adds two 1D histograms
     *
     * @returns a+b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram1D * add ( const std::string & nameAndLabel, const IHistogram1D & a, const IHistogram1D & b ) = 0;
    /**
     * Subtracts two 1D histograms
     *
     * @returns a-b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram1D * sub ( const std::string & nameAndLabel, const IHistogram1D & a, const IHistogram1D & b ) = 0;
    /**
     * Multiplies two 1D histograms
     *
     * @returns a*b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram1D * mul ( const std::string & nameAndLabel, const IHistogram1D & a, const IHistogram1D & b ) = 0;
    /**
     * Divides two 1D histograms
     *
     * @returns a/b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram1D * div ( const std::string & nameAndLabel, const IHistogram1D & a, const IHistogram1D & b ) = 0;
    
    // 2D Histograms
    /**
     * Adds two 2D histograms
     *
     * @returns a+b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram2D * add ( const std::string & nameAndLabel, const IHistogram2D & a, const IHistogram2D & b ) = 0;
    /**
     * Subtracts two 2D histograms
     *
     * @returns a-b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram2D * sub ( const std::string & nameAndLabel, const IHistogram2D & a, const IHistogram2D & b ) = 0;
    /**
     * Multiplies two 2D histograms
     *
     * @returns a*b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram2D * mul ( const std::string & nameAndLabel, const IHistogram2D & a, const IHistogram2D & b ) = 0;
    /**
     * Divides two 2D histograms
     *
     * @returns a/b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram2D * div ( const std::string & nameAndLabel, const IHistogram2D & a, const IHistogram2D & b ) = 0;
    // 3D Histograms
    /**
     * Adds two 3D histograms
     *
     * @returns a+b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram3D * add ( const std::string & nameAndLabel, const IHistogram3D & a, const IHistogram3D & b ) = 0;
    /**
     * Subtracts two 3D histograms
     *
     * @returns a-b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram3D * sub ( const std::string & nameAndLabel, const IHistogram3D & a, const IHistogram3D & b ) = 0;
    /**
     * Multiplies two 3D histograms
     *
     * @returns a*b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram3D * mul ( const std::string & nameAndLabel, const IHistogram3D & a, const IHistogram3D & b ) = 0;
    /**
     * Divides two 3D histograms
     *
     * @returns a/b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual IHistogram3D * div ( const std::string & nameAndLabel, const IHistogram3D & a, const IHistogram3D & b ) = 0;
      
    // 2D Projections
   /**
	 * Create a projection from a 2D histogram parallel to the X axis.
	 * Equivalent to <tt>sliceX(UNDERFLOW_BIN,OVERFLOW_BIN)</tt>.
	 */
   virtual IHistogram1D * projectionX ( const std::string & nameAndLabel, const IHistogram2D & h ) = 0;
	/**
	 * Create a projection from a 2D histogram parallel to the Y axis.
	 * Equivalent to <tt>sliceY(UNDERFLOW_BIN,OVERFLOW_BIN)</tt>.
	 */
   virtual IHistogram1D * projectionY ( const std::string & nameAndLabel, const IHistogram2D & h ) = 0;
	/**
	 * Slice parallel to the Y axis from a 2D histogram at bin indexY and one bin wide.
	 * Equivalent to <tt>sliceX(indexY,indexY)</tt>.
	 */
   virtual IHistogram1D * sliceX ( const std::string & nameAndLabel, const IHistogram2D & h, int indexY ) = 0;
	/**
	 * Slice parallel to the X axis from a 2D histogram at bin indexX and one bin wide.
	 * Equivalent to <tt>sliceY(indexX,indexX)</tt>.
	 */
   virtual IHistogram1D * sliceY ( const std::string & nameAndLabel, const IHistogram2D & h, int indexX ) = 0;
	/**
	 * Create a slice parallel to the X axis from a 2D histogram, 
	 * between "indexY1" and "indexY2" (inclusive).
	 * The returned IHistogram1D represents an instantaneous snapshot of the
	 * histogram at the time the slice was created.
	 */ 
   virtual IHistogram1D * sliceX ( const std::string & nameAndLabel, const IHistogram2D & h, int indexY1, int indexY2 ) = 0;
	/**
	 * Create a slice parallel to the Y axis from a 2D histogram, 
	 * between "indexX1" and "indexX2" (inclusive).
	 * The returned IHistogram1D represents an instantaneous snapshot of the
	 * histogram at the time the slice was created.
	 */ 
   virtual IHistogram1D * sliceY ( const std::string & nameAndLabel, const IHistogram2D & h, int indexX1, int indexX2 ) = 0;
        /**
         * Create a projection parallel to the X axis from a 3D histogram.
         * Equivalent to <tt>sliceXY(UNDERFLOW_BIN,OVERFLOW_BIN)</tt>.
         */
   virtual IHistogram2D * projectionXY ( const std::string & nameAndLabel, const IHistogram3D & h ) = 0;
        /**
         * Create a projection parallel to the Z axis from a 3D histogram.
         * Equivalent to <tt>sliceXZ(UNDERFLOW_BIN,OVERFLOW_BIN)</tt>.
         */
   virtual IHistogram2D * projectionXZ ( const std::string & nameAndLabel, const IHistogram3D & h ) = 0;
        /**
         * Create a projection parallel to the Y axis from a 3D histogram.
         * Equivalent to <tt>sliceYZ(UNDERFLOW_BIN,OVERFLOW_BIN)</tt>.
         */
   virtual IHistogram2D * projectionYZ ( const std::string & nameAndLabel, const IHistogram3D & h ) = 0;
        /**
         * Create a slice perpendicular to the Z axis from a 3D histogram,
         * between "indexZ1" and "indexZ2" (inclusive).
         * The returned IHistogram2D represents an instantaneous snapshot of the
         * histogram at the time the slice was created.
         * The X axis of the returned histogram corresponds to the X axis of this histogram.
         * The Y axis of the returned histogram corresponds to the Y axis of this histogram.
         */
   virtual IHistogram2D * sliceXY ( const std::string & nameAndLabel, const IHistogram3D & h, int indexZ1, int indexZ2 ) = 0;
        /**
         * Create a slice perpendicular to the Y axis from a 3D histogram,
         * between "indexY1" and "indexY2" (inclusive).
         * The returned IHistogram2D represents an instantaneous snapshot of the
         * histogram at the time the slice was created.
         * The X axis of the returned histogram corresponds to the X axis of this histogram.
         * The Y axis of the returned histogram corresponds to the Z axis of this histogram.
         */
   virtual IHistogram2D * sliceXZ ( const std::string & nameAndLabel, const IHistogram3D & h, int indexY1, int indexY2 ) = 0;
        /**
         * Create a slice perpendicular to the X axis from a 3D histogram,
         * between "indexX1" and "indexX2" (inclusive).
         * The returned IHistogram2D represents an instantaneous snapshot of the
         * histogram at the time the slice was created.
         * The X axis of the returned histogram corresponds to the Y axis of this histogram.
         * The Y axis of the returned histogram corresponds to the Z axis of this histogram.
         */
   virtual IHistogram2D * sliceYZ ( const std::string & nameAndLabel, const IHistogram3D & h, int indexX1, int indexX2 ) = 0;
};
#endif /* ifndef AIDA_IHISTOGRAMFACTORY_H */ 

