
// This file has been automatically converted from IAnalysisFactory.java 

#ifndef AIDA_IANALYSISFACTORY_H 
#define AIDA_IANALYSISFACTORY_H 1
/**
 * The "master" factory from which other factories are obtained.
 * Typically accessed by: 
 * <pre>
 *   IAnalysisFactory af = (IAnalysisFactory) ServiceManager.create("hep.aida.IAnalysisFactory");
 * </pre>
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version $Id: IAnalysisFactory.h,v 1.5 2001/11/23 15:26:24 barrand Exp $
 */
// C++ IAnalysisFactory* af = AIDA_createAnalysisFactory();


// type ahead declarations 
class ICloudFactory;
class ITupleFactory;
class ITreeFactory;
class IHistogramFactory;
class IFunctionFactory;
class ITree;
class IEvaluatorFactory;
class IFilterFactory;
class IPlotterFactory;

class IAnalysisFactory  {

public:

   virtual ~IAnalysisFactory () { /* nop */; }
    /**
     * Create an ITreeFactory.
     */
   virtual ITreeFactory * createTreeFactory (  ) = 0;
    /**
     * Create an IHistogramFactory.
     * @param tree The ITree which created histograms will be associated
     */
   virtual IHistogramFactory * createHistogramFactory ( ITree & tree ) = 0;
    /**
     * Create an ICloudFactory.
     * @param tree The ITree which created clouds will be associated
     */
   virtual ICloudFactory * createCloudFactory ( ITree & tree ) = 0;
    /**
     * Create an ITupleFactory.
     * @param tree The ITree which created tuples will be associated
     */
   virtual ITupleFactory * createTupleFactory ( ITree & tree ) = 0;
    /**
     * Create an IFunctionFactory.
     * @param tree The ITree which created functions will be associated
     */
   virtual IFunctionFactory * createFunctionFactory ( ITree & tree ) = 0;
    /**
     * Create an IEvaluatorFactory.
     */
   virtual IEvaluatorFactory * createEvaluatorFactory ( ) = 0;
    /**
     * Create an IFilterFactory.
     */
   virtual IFilterFactory * createFilterFactory ( ) = 0;
    /**
     * Create an IPlotterFactory.
     * Due to the fact that the plotter factory may activate a GUI 
     * tookit, the process arguments (passed to the main function)
     * could be passed to the plotter factory. Most of the GUI
     * toolkit ask for them (Motif, gtk, Qt, FLTK).
     */
   virtual IPlotterFactory * createPlotterFactory (int argc = 0,
						   char** args = 0 ) = 0;
};


/**
 * To hook an AIDA implementation.
 * For example :
 *   #include <AIDA/IAnalysisFactory.h>          
 *   ...
 *   IAnalysisFactory* analysisFactory = AIDA_createAnalysisFactory();
 *   ...
 */

extern "C" {
IAnalysisFactory* AIDA_createAnalysisFactory();
}


#endif /* ifndef AIDA_IANALYSISFACTORY_H */ 

