#ifndef AIDA_IANNOTATION_H 
#define AIDA_IANNOTATION_H 1

#include <string>

/**
 * Open questions about IAnnotation (GB) :
 *  What is it for ? 
 *  What is the definition for it ?
 *  Do we have, for coherency, to put a annotation() method on all "data classes"
 * (IHistogram, ICloud, IFunction, ITuple) ? 
 *  Do we assume a one to one relationship ?
 *  When storing a "data object" (IHistogram, IFunction, ...)
 * do we have to store/retrieve the associated IAnnotation ?
 * 
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version $Id: IAnnotation.h,v 1.5 2001/11/26 09:03:26 barrand Exp $
 */

class IAnnotation  {
public:
  virtual ~IAnnotation () { /* nop */; }
public:
  /**
   * Add a key/value pair with a given visibility.
   * What visibility is for ?
   * If key alredy exists, override its value ?
   */
  virtual bool addItem( const std::string & key, 
			const std::string & value,
			bool visibility = true) = 0;

  /**
   * Remove the item indicated by a given key
   */
  virtual bool removeItem( const std::string & key )= 0;

  /**
   * Retrieve the value for a given key
   */
  virtual const std::string & value( const std::string & key) const = 0;

  /**
   * Retrieve the visibility for a given key
   */
  virtual bool visibility( const std::string & key) const = 0;

  /**
   * Set value for a given key
   */
  virtual void setValue( const std::string & key, const std::string& value) = 0;

  /**
   * Set visibility for a given key
   */
  virtual void setVisibility( const std::string & key, bool visibility) = 0;

  /// For "iterative" access to the objects ...
  /// in future releases we might have an iterator ...

  /// Get the number of items in the Annotation
  virtual int size() const = 0;

  /// Individual access to the Annotation-items
  /// This probably should be replaced by a proper iterator
  /// returning a IAnnotationItem object ...
  virtual const std::string & key(int index) const = 0;
  virtual const std::string & value(int index) const = 0;
  virtual bool visibility(int index) const = 0;

  /// Reset contents
  virtual void reset() = 0;

};

#endif /* ifndef AIDA_IANNOTATION_H */

