
// This file has been automatically converted from IAxis.java 

#ifndef AIDA_IAXIS_H 
#define AIDA_IAXIS_H 1

/**
 * An IAxis represents a binned histogram axis. A 1D Histogram would have
 * one Axis representing the X axis, while a 2D Histogram would have two
 * axes representing the X and Y Axis. 
 *
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version $Id: IAxis.h,v 1.4 2001/08/08 08:17:15 pfeiffer Exp $
 */


class IAxis {

public:

   virtual ~IAxis() { /* nop */; }
	/**
	 *  Lower axis edge.
	 */
   virtual double lowerEdge (  ) const = 0;
	/**
	 *  Upper axis edge.
	 */
   virtual double upperEdge (  ) const = 0;
	/** 
	 * The number of bins (excluding underflow and overflow) on the axis.
	 */
   virtual int bins (  ) const = 0;
	/**
	 * Lower edge of the specified bin.
	 * @param index Bin number (0...bins()-1) or OVERFLOW or UNDERFLOW.
	 * @return the lower edge of the bin; for the underflow bin this is <tt>Double.NEGATIVE_INFINITY</tt>.
	 */
   virtual double binLowerEdge ( int index ) const = 0;
	/**
	 * Upper edge of the specified bin.
	 * @param index Bin number (0...bins()-1) or OVERFLOW or UNDERFLOW.
	 * @return the upper edge of the bin; for the overflow bin this is <tt>Double.POSITIVE_INFINITY</tt>.
	 */ 
   virtual double binUpperEdge ( int index ) const = 0;
	/**
	 * Width of the bin specified.
	 * @param index Bin number (0...bins()-1) or OVERFLOW or UNDERFLOW.
	 */ 
   virtual double binWidth ( int index ) const = 0;
	/**
	 * Converts a coordinate on the axis to a bin number. If the coordinate
	 * is < lowerEdge returns UNDERFLOW, and if the coordinate is >= 
	 * upperEdge returns OVERFLOW.
	 */
   virtual int coordToIndex ( double coord ) const = 0;
};
#endif /* ifndef AIDA_IAXIS_H */ 

