
#ifndef AIDA_IEVALUATORFACTORY_H 
#define AIDA_IEVALUATORFACTORY_H 1

/**
 * Creation of Evaluators
 * 
 * @author Guy Barrand
 */

// includes 
#include <string>

// type ahead declarations 
class ITuple;
class IEvaluator;

class IEvaluatorFactory {
    
public:
  virtual ~IEvaluatorFactory () { /* nop */; }

  /**
   * Create an evaluator that uses some interpreter.
   * @param ITuple : tuple to work with.
   * @param script : the script has the syntax of one C statement.
   *                 Variables are assumed to be labels of tuple columns.
   *                 Standard C math function are authorized.
   *                 for example "2*A*sin(B)"
   * @returns the requested evaluator.
   */
  virtual IEvaluator* createScripted(const ITuple&,
				     const std::string & script) = 0;

};

#endif /* AIDA_IEVALUATORFACTORY_H */
