
#ifndef AIDA_IFUNCTIONFACTORY_H 
#define AIDA_IFUNCTIONFACTORY_H 1

/**
 * Creation of Functions
 * 
 * @author Andreas Pfeiffer, Guy Barrand
 * @version $Id: IFunctionFactory.h,v 1.4 2001/11/12 10:03:05 barrand Exp $
 */

// includes 
#include <string>
#include <vector>

// type ahead declarations 
class IFunction;
class IFitFunction;

class IFunctionFactory {
    
public:
  virtual ~IFunctionFactory () { /* nop */; }

  /**
   * @param type  A string to describe the type of the function: e.g.,
   *              "G"  or "Gaussian"    for gaussian, 
   *              "E"  or "Exponential" for exponential,
   *              "P"  or "Polynomial"  for polynomial(n) (see below for specifying the degree),
   *              "BW" or "BreitWigner" for Breit-Wigner function
   * @param parameters The parameters for the function. 
   *                   For the Polynomial the size of the vector determines the degree of it.
   * @returns the requested function.
   */
  virtual IFunction * create(const std::string & name, const std::string & label, 
			     const std::string & type, 
			     const std::vector<double> & parameterValues, 
			     const std::string & parameterNames = "") = 0;

  virtual IFunction * createScripted(const std::string & name, const std::string & label, 
				     const std::string & script) = 0;

  virtual IFitFunction * createFit(const std::string & name, const std::string & label, 
				   const std::string & type,
				   const std::string & options) = 0;

};

#endif /* AIDA_IFUNCTIONFACTORY_H */
