
// This file has been automatically converted from IHistogram3D.java 

#ifndef AIDA_IHISTOGRAM3D_H 
#define AIDA_IHISTOGRAM3D_H 1

/**
 * User level interface to 3D Histogram.
 *
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version $Id: IHistogram3D.h,v 1.8 2001/11/12 09:31:38 barrand Exp $
 */

// type ahead declarations 
class IAxis;

// base class(es)  
#include "AIDA/IHistogram.h"


class IHistogram3D  :  virtual public IHistogram {

public:

   virtual ~IHistogram3D () { /* nop */; }
        /**
         * Fill the histogram with specified weight.
         */
   virtual bool fill ( double x, double y, double z, double weight = 1. ) = 0;
	/**
	 * Centre along x, y or z of the corresponding bin. As this may not be the simple average 
	 * between upper and lower edge of the bin, it has been moved from IAxis to here.
	 * @param index the x (and y, z, resp.) bin number (0...Nx-1) or OVERFLOW or UNDERFLOW.
	 */
   virtual double binCentreX ( int indexX ) const = 0;
   virtual double binCentreY ( int indexY ) const = 0;
   virtual double binCentreZ ( int indexZ ) const = 0;
        /**
         * The number of entries (ie the number of times fill was called for this bin).
         * @param indexX the x bin number (0...Nx-1) or OVERFLOW or UNDERFLOW.
         * @param indexY the y bin number (0...Ny-1) or OVERFLOW or UNDERFLOW.
	 * @param indexZ the z bin number (0...Nz-1) or OVERFLOW or UNDERFLOW.
         */
   virtual int binEntries ( int indexX, int indexY, int indexZ ) const = 0;
        /**
         * Equivalent to <tt>projectionXY().binEntriesX(indexX)</tt>.
         */
   virtual int binEntriesX ( int indexX ) const = 0;
        /**
         * Equivalent to <tt>projectionXY().binEntriesY(indexY)</tt>.
         */
   virtual int binEntriesY ( int indexY ) const = 0;
        /**
         * Equivalent to <tt>projectionXZ().binEntriesZ(indexZ)</tt>.
         */
   virtual int binEntriesZ ( int indexZ ) const = 0;
        /**
         * Total height of the corresponding bin (ie the sum of the weights in this bin).
         * @param indexX the x bin number (0...Nx-1) or OVERFLOW or UNDERFLOW.
         * @param indexY the y bin number (0...Ny-1) or OVERFLOW or UNDERFLOW.
	 * @param indexZ the z bin number (0...Nz-1) or OVERFLOW or UNDERFLOW.
         */
   virtual double binHeight ( int indexX, int indexY, int indexZ ) const = 0;
        /**
         * Equivalent to <tt>projectionXY().binHeightX(indexX)</tt>.
         */
   virtual double binHeightX ( int indexX ) const = 0;
        /**
         * Equivalent to <tt>projectionXY().binHeightY(indexY)</tt>.
         */
   virtual double binHeightY ( int indexY ) const = 0;
        /**
         * Equivalent to <tt>projectionXZ().binHeightY(indexZ)</tt>.
         */
   virtual double binHeightZ ( int indexZ ) const = 0;
        /**
         * The error on this bin.
         * @param indexX the x bin number (0...Nx-1) or OVERFLOW or UNDERFLOW.
         * @param indexY the y bin number (0...Ny-1) or OVERFLOW or UNDERFLOW.
	 * @param indexZ the z bin number (0...Nz-1) or OVERFLOW or UNDERFLOW.
         */
   virtual double binError ( int indexX, int indexY, int indexZ ) const = 0;
        /**
         *  Returns the mean of the histogram, as calculated on filling-time projected on the X axis.
         */
   virtual double meanX (  ) const = 0;
        /**
         *  Returns the mean of the histogram, as calculated on filling-time projected on the Y axis.
         */
   virtual double meanY (  ) const = 0;
        /**
         *  Returns the mean of the histogram, as calculated on filling-time projected on the Z axis.
         */
   virtual double meanZ (  ) const = 0;
        /**
         * Returns the rms of the histogram as calculated on filling-time projected on the X axis.
         */
   virtual double rmsX (  ) const = 0;
        /**
         * Returns the rms of the histogram as calculated on filling-time projected on the Y axis.
         */
   virtual double rmsY (  ) const = 0;
        /**
         * Returns the rms of the histogram as calculated on filling-time projected on the Z axis.
         */
   virtual double rmsZ (  ) const = 0;
        /**
         * Return the X axis.
         */
   virtual const IAxis & xAxis (  ) const = 0;
        /**
         * Return the Y axis.
         */
   virtual const IAxis & yAxis (  ) const = 0;
        /**
         * Return the Z axis.
         */
   virtual const IAxis & zAxis (  ) const = 0;
        /**
         * Convenience method, equivalent to <tt>xAxis().coordToIndex(coord)</tt>.
         * @see IAxis#coordToIndex(double)
         */
   virtual int coordToIndexX ( double coordX ) const = 0;
        /**
         * Convenience method, equivalent to <tt>yAxis().coordToIndex(coord)</tt>.
         * @see IAxis#coordToIndex(double)
         */
   virtual int coordToIndexY ( double coordY ) const = 0;
        /**
         * Convenience method, equivalent to <tt>zAxis().coordToIndex(coord)</tt>.
         * @see IAxis#coordToIndex(double)
         */
   virtual int coordToIndexZ ( double coordZ ) const = 0;

    /**
     * Modifies this histogram by adding the contents of h to it.
     *
     * @param h The histogram to be added to this histogram
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual bool add ( const IHistogram3D & h ) = 0;
};


#endif /* ifndef AIDA_IHISTOGRAM3D_H */ 

