package hep.aida;

/**
 * User level interface to 2D Histogram.
 * 
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version $Id: IHistogram2D.java,v 1.5 2001/11/07 22:41:00 tonyj Exp $
 */
public interface IHistogram2D extends IHistogram {
	/**
	 * Fill the histogram with weight 1.
	 */
	public void fill( double x, double y);
	/**
	 * Fill the histogram with specified weight.
	 */
	public void fill( double x, double y, double weight);
	/**
	 * Centre along x or y of the corresponding bin. As this may not be the simple average 
	 * between upper and lower edge of the bin, it has been moved from IAxis to here.
	 * @param index the x (or y) bin number (0...Nx-1) or OVERFLOW or UNDERFLOW.
	 */
   public double binCentreX ( int indexX );
   public double binCentreY ( int indexY );
	/**
	 * The number of entries (ie the number of times fill was called for this bin).
	 * @param indexX the x bin number (0...Nx-1) or OVERFLOW or UNDERFLOW.
	 * @param indexY the y bin number (0...Ny-1) or OVERFLOW or UNDERFLOW.
	 */
	public int binEntries(int indexX, int indexY );
	/**
	 * Equivalent to <tt>projectionX().binEntries(indexX)</tt>.
	 */
	public int binEntriesX(int indexX);
	/**
	 * Equivalent to <tt>projectionY().binEntries(indexY)</tt>.
	 */
	public int binEntriesY(int indexY);
	/**
	 * Total height of the corresponding bin (ie the sum of the weights in this bin).
	 * @param indexX the x bin number (0...Nx-1) or OVERFLOW or UNDERFLOW.
	 * @param indexY the y bin number (0...Ny-1) or OVERFLOW or UNDERFLOW.
	 */ 
	public double binHeight(int indexX,int indexY );
	/**
	 * Equivalent to <tt>projectionX().binHeight(indexX)</tt>.
	 */
	public double binHeightX(int indexX);
	/**
	 * Equivalent to <tt>projectionY().binHeight(indexY)</tt>.
	 */
	public double binHeightY(int indexY);
	/**
	 * The error on this bin.
	 * @param indexX the x bin number (0...Nx-1) or OVERFLOW or UNDERFLOW.
	 * @param indexY the y bin number (0...Ny-1) or OVERFLOW or UNDERFLOW.
	 */ 
	public double binError(int indexX,int indexY );
	/**
	 *  Returns the mean of the histogram, as calculated on filling-time projected on the X axis.
	 */ 
	public double meanX();
	/**
	 *  Returns the mean of the histogram, as calculated on filling-time projected on the Y axis.
	 */ 
	public double meanY();
	/**
	 * Returns the rms of the histogram as calculated on filling-time projected on the X axis.
	 */
	public double rmsX();
	/**
	 * Returns the rms of the histogram as calculated on filling-time projected on the Y axis.
	 */
	public double rmsY();
	/**
	 * Return the X axis.
	 */
	public IAxis xAxis();
	/**
	 * Return the Y axis.
	 */
	public IAxis yAxis();
	/**
	 * Convenience method, equivalent to <tt>xAxis().coordToIndex(coord)</tt>.
	 * @see IAxis#coordToIndex(double)
	 */  
	public int coordToIndexX(double coordX);
	/**
	 * Convenience method, equivalent to <tt>yAxis().coordToIndex(coord)</tt>.
	 * @see IAxis#coordToIndex(double)
	 */  
	public int coordToIndexY(double coordY);
        
    /**
     * Modifies this histogram by adding the contents of h to it.
     *
     * @param h The histogram to be added to this histogram
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
    public void add(IHistogram2D h) throws IllegalArgumentException; 
}
