package hep.aida;

/**
 * User level interface to 3D Histogram.
 *
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version $Id: IHistogram3D.java,v 1.5 2001/11/07 22:41:00 tonyj Exp $
 */
public interface IHistogram3D extends IHistogram {
        /**
         * Fill the histogram with weight 1.
         */
    public void fill( double x, double y, double z);
        /**
         * Fill the histogram with specified weight.
         */
    public void fill( double x, double y, double z, double weight);
	/**
	 * Centre along x, y or z of the corresponding bin. As this may not be the simple average 
	 * between upper and lower edge of the bin, it has been moved from IAxis to here.
	 * @param index the x (and y, z, resp.) bin number (0...Nx-1) or OVERFLOW or UNDERFLOW.
	 */
   public double binCentreX ( int indexX );
   public double binCentreY ( int indexY );
   public double binCentreZ ( int indexZ );
        /**
         * The number of entries (ie the number of times fill was called for this bin).
         * @param indexX the x bin number (0...Nx-1) or OVERFLOW or UNDERFLOW.
         * @param indexY the y bin number (0...Ny-1) or OVERFLOW or UNDERFLOW.
	 * @param indexZ the z bin number (0...Nz-1) or OVERFLOW or UNDERFLOW.
         */
    public int binEntries(int indexX, int indexY, int indexZ);
        /**
         * Equivalent to <tt>projectionXY().binEntriesX(indexX)</tt>.
         */
    public int binEntriesX(int indexX);
        /**
         * Equivalent to <tt>projectionXY().binEntriesY(indexY)</tt>.
         */
    public int binEntriesY(int indexY);
        /**
         * Equivalent to <tt>projectionXZ().binEntriesY(indexZ)</tt>.
         */
    public int binEntriesZ(int indexZ);
        /**
         * Total height of the corresponding bin (ie the sum of the weights in this bin).
         * @param indexX the x bin number (0...Nx-1) or OVERFLOW or UNDERFLOW.
         * @param indexY the y bin number (0...Ny-1) or OVERFLOW or UNDERFLOW.
	 * @param indexZ the z bin number (0...Nz-1) or OVERFLOW or UNDERFLOW.
         */
    public double binHeight(int indexX, int indexY, int indexZ);
        /**
         * Equivalent to <tt>projectionXY().binHeightX(indexX)</tt>.
         */
    public double binHeightX(int indexX);
        /**
         * Equivalent to <tt>projectionXY().binHeightY(indexY)</tt>.
         */
    public double binHeightY(int indexY);
        /**
         * Equivalent to <tt>projectionXZ().binHeightY(indexZ)</tt>.
         */
    public double binHeightZ(int indexZ);
        /**
         * The error on this bin.
         * @param indexX the x bin number (0...Nx-1) or OVERFLOW or UNDERFLOW.
         * @param indexY the y bin number (0...Ny-1) or OVERFLOW or UNDERFLOW.
	 * @param indexZ the z bin number (0...Nz-1) or OVERFLOW or UNDERFLOW.
         */
    public double binError(int indexX, int indexY, int indexZ);
        /**
         *  Returns the mean of the histogram, as calculated on filling-time projected on the X axis.
         */
    public double meanX();
        /**
         *  Returns the mean of the histogram, as calculated on filling-time projected on the Y axis.
         */
    public double meanY();
        /**
         *  Returns the mean of the histogram, as calculated on filling-time projected on the Z axis.
         */
    public double meanZ();
        /**
         * Returns the rms of the histogram as calculated on filling-time projected on the X axis.
         */
    public double rmsX();
        /**
         * Returns the rms of the histogram as calculated on filling-time projected on the Y axis.
         */
    public double rmsY();
        /**
         * Returns the rms of the histogram as calculated on filling-time projected on the Z axis.
         */
    public double rmsZ();
        /**
         * Return the X axis.
         */
    public IAxis xAxis();
        /**
         * Return the Y axis.
         */
    public IAxis yAxis();
        /**
         * Return the Z axis.
         */
    public IAxis zAxis();
        /**
         * Convenience method, equivalent to <tt>xAxis().coordToIndex(coord)</tt>.
         * @see IAxis#coordToIndex(double)
         */
    public int coordToIndexX(double coordX);
        /**
         * Convenience method, equivalent to <tt>yAxis().coordToIndex(coord)</tt>.
         * @see IAxis#coordToIndex(double)
         */
    public int coordToIndexY(double coordY);
        /**
         * Convenience method, equivalent to <tt>zAxis().coordToIndex(coord)</tt>.
         * @see IAxis#coordToIndex(double)
         */
    public int coordToIndexZ(double coordZ);

    /**
     * Modifies this histogram by adding the contents of h to it.
     *
     * @param h The histogram to be added to this histogram
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
    public void add(IHistogram3D h) throws IllegalArgumentException;
}


