package hep.aida;

import java.io.*;

/**
 * The creator of trees.
 *
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version $Id: ITreeFactory.java,v 1.4 2001/07/24 13:24:04 duns Exp $
 */
public interface ITreeFactory 
{
    /**
     * Creates an in memory tree. Cannot be associated with a file.
     */
    public ITree create();
    /**
     * Creates a new tree and associates it with a store.
     * The store is assumed to be read/write.
     * The store will be created if it does not exist.
     * @param storeName The name for the store.
     * @throws IOException if the store already exists
     */ 
    public ITree create(String storeName) throws IllegalArgumentException, IOException;
    /**
     * Creates a new tree and associates it with a store.
     * The store is assumed to be read/write.
     * The store will be created if it does not exist.
     * @param storeName The name for the store.
     * @param storeType Implementation specific string, may control store type
     */    
    public ITree create(String storeName, String storeType) throws IllegalArgumentException, IOException;
    /**
     * Creates a new tree and associates it with a store.
     * @param readOnly If true the store is opened readonly, an exception if it does not exist
     * @param exists If true the file must exist, if false the file will be created
     * @param storeName The name of the store
     */
    public ITree create(String storeName, boolean readOnly, boolean exists) throws IllegalArgumentException, IOException;
    /**
     * Creates a new tree and associates it with a store.
     * @param readOnly If true the store is opened readonly, an exception if it does not exist
     * @param exists If true the file must exist, if false the file will be created
     * @param storeName The name of the store
     */
    public ITree create(String storeName, boolean readOnly, boolean exists, String storeType) throws IllegalArgumentException, IOException;
}
