
// This file has been automatically converted from ITree.java 

#ifndef AIDA_ITREE_H 
#define AIDA_ITREE_H 1


/**
 * User level interface to a Tree.
 *
 * All paths follow unix convention .., ., /, 
 * backslash is the escape character.
 * Relative paths are allowed.
 *
 * NOTE: - this tree keeps a current position (pwd) within the tree.
 * This may be implemented on a per-thread basis.
 *
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version $Id: ITree.h,v 1.8 2001/08/09 08:01:49 pfeiffer Exp $
 */

// includes 
#include <string>

// type ahead declarations 
class IManagedObject;
class Iterator;
class ITree;


class ITree  {

public:

   virtual ~ITree () { /* nop */; }
    
   virtual IManagedObject * find ( const std::string & path ) = 0;
    
   virtual bool cd ( const std::string & path ) = 0;
   virtual const std::string & pwd (  ) const = 0;
    
    // All list methods return all managed objects,
    // including folders, but not symlinks
   virtual Iterator * ls ( const std::string & path = ".", bool recursive = false) const = 0;
    
    // path = subdir.../dir
    // throws exception if 
    // - subdir does not exist
    // - dir does exist
    // - subdir is not a folder
   virtual bool mkdir ( const std::string & path ) = 0;
    
    // path = subdir.../dir
    // throws exception if 
    // - subdir is not a folder
   virtual bool mkdirs ( const std::string & path ) = 0;
    
   virtual bool rmdir ( const std::string & path ) = 0;
    // removing a mount point should commit, close and delete the tree object    
   virtual bool rm ( const std::string & path ) = 0;
   virtual const std::string & findPath ( const IManagedObject & object ) = 0;
   virtual bool mv ( const std::string & oldPath, const std::string & newPath ) = 0;

   virtual bool commit (  ) = 0;
   virtual bool cp ( const std::string & oldPath, const std::string & newPath, bool recursive = false ) = 0;
   virtual bool symlink ( const std::string & path, const std::string & alias ) = 0;
    // a tree can only be mounted once
    /**
     * Mounts a tree within another (target) tree.
     * Example:
     * <pre>
     *     target.mount("/home/tonyj",tree,"/");
     * </pre>
     * @param path The path in the target tree
     * @param tree The tree to mount within the target tree
     * @param treePath the mount point within the tree to be mounted.
     */
   virtual bool mount ( const std::string & path, const ITree & tree, const std::string & treePath ) = 0;
   virtual bool unmount ( const std::string & path ) = 0;
    // close calls commit before closing
   virtual bool close (  ) = 0;
};
#endif /* ifndef AIDA_ITREE_H */ 

