
// This file has been automatically converted from IHistogram.java 

#ifndef AIDA_IHISTOGRAM_H 
#define AIDA_IHISTOGRAM_H 1

/**
 * User level interface to Histogram.
 *
 * - NOTE: fillable or non-fillable
 *
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version $Id: IHistogram.h,v 1.10 2001/11/29 07:38:44 barrand Exp $
 */

// includes 

// type ahead declarations 
class IAnnotation;

// base class(es)  
#include "AIDA/IConstants.h"

class IHistogram  :  virtual public IConstants {

public:

  virtual ~IHistogram () { /* nop */; }

  /**
   * Label of the histogram
   *
   * @returns null if label was not set.
   */ 
  virtual const std::string & label (  ) const = 0;
	
  virtual bool setLabel ( const std::string & label ) = 0;
  /**
   * Title of the histogram (will be set only in the constructor).
   * @deprecated please use label()
   */ 
  virtual const std::string & title (  ) const = 0;
  /**
   * It is assumed that the histogram manages its IAnnotation.
   * Then someone don't have to delete explicitly the returned object.
   * When exception will be used, we have to return a reference.
   */
  virtual IAnnotation * annotation (  ) = 0;
  /**
   * Returns 1 for one-dimensional histograms, 2 for two-dimensional histograms, and so on.
   */ 
  virtual int dimension (  ) const = 0;
  /**
   * Reset contents; as if just constructed.
   */
  virtual bool reset (  ) = 0;
  /**
   * Number of in-range entries in the histogram.
   */ 
  virtual int entries (  ) const = 0;
  /**
   * Number of all entries in all (both in-range and under/overflow) bins in the histogram.
   */ 
  virtual int allEntries (  ) const = 0;
  /**
   * Number of under and overflow entries in the histogram.
   */ 
  virtual int extraEntries (  ) const = 0;
  /** 
   * Number of equivalent entries.
   * @return <tt>SUM[ weight ] ^ 2 / SUM[ weight^2 ]</tt>.
   */ 
  virtual double equivalentBinEntries (  ) const = 0;
  /**
   * Sum of in-range bin heights in the histogram.
   */ 
  virtual double sumBinHeights (  ) const = 0;
  /**
   * Sum of all (both in-range and under/overflow) bin heights in the histogram.
   */ 
  virtual double sumAllBinHeights (  ) const = 0;
  /**
   * Sum of under/overflow bin heights in the histogram.
   */ 
  virtual double sumExtraBinHeights (  ) const = 0;
  /**
   * Minimum height of in-range bins in the histogram.
   */ 
  virtual double minBinHeight (  ) const = 0;
  /**
   * Maximum height of in-range bins in the histogram.
   */ 
  virtual double maxBinHeight (  ) const = 0;


};
#endif /* ifndef AIDA_IHISTOGRAM_H */ 

