
// This file has been automatically converted from IHistogram1D.java 

#ifndef AIDA_IHISTOGRAM1D_H 
#define AIDA_IHISTOGRAM1D_H 1

/**
 * User level interface to 1D Histogram.
 * 
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version $Id: IHistogram1D.h,v 1.11 2001/11/29 07:38:44 barrand Exp $
 */

// type ahead declarations 
class IAxis;
class IFitFunction;

// base class(es)  
#include "AIDA/IHistogram.h"


class IHistogram1D  :  virtual public IHistogram {

public:

   virtual ~IHistogram1D () { /* nop */; }
	/**
	 * Fill histogram with specified weight.
	 */
   virtual bool fill ( double x, double weight = 1. ) = 0;
	/**
	 * Centre of the corresponding bin. As this may not be the simple average 
	 * between upper and lower edge of the bin, it has been moved from IAxis to here.
	 * @param index the bin number (0...N-1) or OVERFLOW or UNDERFLOW.
	 */
   virtual double binCentre ( int index ) const = 0;
	/**
	 * Total height of the corresponding bin (ie the sum of the weights in this bin).
	 * @param index the bin number (0...N-1) or OVERFLOW or UNDERFLOW.
	 */ 
   virtual int binEntries ( int index ) const = 0;
	/**
	 * Total height of the corresponding bin (ie the sum of the weights in this bin).
	 * @param index the bin number (0...N-1) or OVERFLOW or UNDERFLOW.
	 */ 
   virtual double binHeight ( int index ) const = 0;
	/**
	 * The error on this bin.
	 * @param index the bin number (0...N-1) or OVERFLOW or UNDERFLOW.
	 */ 
   virtual double binError ( int index ) const = 0;
	/**
	 * Returns the mean of the whole histogram as calculated on filling-time.
	 */ 
   virtual double mean (  ) const = 0;
	/**
	 * Returns the rms of the whole histogram as calculated on filling-time.
	 */ 
   virtual double rms (  ) const = 0;
	/**
	 * Returns the X axis.
	 */
   virtual const IAxis & axis (  ) const = 0;
	/**
	 * Convenience method, equivalent to <tt>axis().coordToIndex(coord)</tt>.
	 * @see IAxis#coordToIndex(double)
	 */
   virtual int coordToIndex ( double coord ) const = 0;

    /**
     * Modifies this histogram by adding the contents of h to it.
     *
     * @param h The histogram to be added to this histogram
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
   virtual bool add ( const IHistogram1D & h ) = 0;
    
};



#endif /* ifndef AIDA_IHISTOGRAM1D_H */ 

