
// This file has been automatically converted from ITuple.java 

#ifndef AIDA_ITUPLE_H 
#define AIDA_ITUPLE_H 1

/**
 * User level interface to a Tuple.
 * 
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version $Id: ITuple.h,v 1.9 2001/11/12 11:32:52 barrand Exp $
 */

// includes 
#include <vector>
#include <string>

// type ahead declarations 
class INtupleEntry;
class IEvaluator;
class IFilter;
class IHistogram1D;
class IHistogram2D;
class IHistogram3D;

class ITuple  {

public:

   virtual ~ITuple () { /* nop */; }
    
    /** The label for this n-tuple */
   virtual const std::string & label (  ) const = 0;
    
    // writing interface, always at the end
   virtual bool fill ( int column, double value ) = 0;
   virtual bool fill ( int column, float  value ) = 0;
   virtual bool fill ( int column, int    value ) = 0;
   virtual bool fill ( int column, short  value ) = 0;
   virtual bool fill ( int column, long   value ) = 0;
   virtual bool fill ( int column, char   value ) = 0;
// unknown parameter 'byte' found:     public void fill(int column, byte value) throws IllegalArgumentException;
   virtual bool fill ( int column, bool   value ) = 0;
   virtual bool fill ( int column, const std::string &  value ) = 0;
   virtual bool fill ( int column, const INtupleEntry & value ) = 0;
    
   virtual bool fill ( const std::vector< double > & values ) = 0;
   virtual bool fill ( const std::vector< float > &  values ) = 0;
    /**
     * Should be called after fill is called for the columns.
     * Unfilled columns will be filled with the default value for that column.
     */
   virtual bool addRow (  ) = 0;
    /**
     * Clears the current (not yet added) row.
     */
   virtual bool resetRow (  ) = 0;
    /**
     * Clears all rows from ntuple?
     */
   virtual bool reset (  ) = 0;
    
    // reading interface
    /**
     * The number of rows currently in the ntuple.
     * @returns -1 if cannot be determined.
     */
   virtual int rows (  ) const = 0;
    /**
     * Positions the read cursor immediately before the first row.
     */
   virtual bool start (  ) = 0;
    /**
     * Skips rows.
     * @param rows number of rows to skip, greater than 0.
     * @returns false if it cannot skip rows.
     */
   virtual bool skip ( int rows ) = 0;
    /**
     * Positions the cursor at the next row.
     * @returns false if there is no next row.
     */
   virtual bool next (  ) = 0;
    /**
     * Convert a name to a column index.
     * Note: C++ version may return -1 if column not found
     * @returns the column number for a given name 
     */
   virtual int findColumn ( const std::string & name ) const = 0;
    
   virtual double getDouble              ( int column ) const = 0;
   virtual float  getFloat               ( int column ) const = 0;
   virtual int    getInt                 ( int column ) const = 0;
   virtual short  getShort               ( int column ) const = 0;
   virtual long   getLong                ( int column ) const = 0;
   virtual char   getChar                ( int column ) const = 0;
// unknown parameter 'byte' found:     public byte getByte(int column) throws ClassCastException;
   virtual bool   getBoolean             ( int column ) const = 0;
   virtual const std::string & getString ( int column ) const = 0;
   virtual INtupleEntry * getObject      ( int column ) const = 0;
    
    /** Number of columns in ntuple */
   virtual int columns (  ) const = 0;
    /** Gets the name of a column from its index */
   virtual const std::string & columnName ( int column ) const = 0;
    /** Returns the type of the column (maybe a String in C++) */
   virtual const std::string & columnType ( int column ) const = 0;

    // values can be approximations, or can return Double.NaN
   virtual double columnMin  ( int column ) = 0;
   virtual double columnMax  ( int column ) = 0;
   virtual double columnMean ( int column ) = 0;
   virtual double columnRms  ( int column ) = 0;
    
    // The following methods are under discussion, and will likely
    // be unimplemented in AIDA 2.x
   //virtual bool setFilter ( const IFilter & filter ) = 0;
   //virtual bool addEvaluator ( const std::string & name, const std::string & type, const IEvaluator & evaluator ) = 0;

  /**
   * Fill an IHistogram1D from the tuple.
   * @param histogram. The histogram to fill.
   * @param evaluator. The evaluator object used to compute the value
   *                   to be filled in the histogram. The value is computed
   *                   from the current row of the tuple.
   * @param filter. The filter object used to accept or not the current row.
   *                If rejected, the histogram is not filled.
   */
   virtual bool project(IHistogram1D& histogram,
			const IEvaluator& evaluatorX,
			const IFilter& filter) = 0;

  /**
   * Fill an IHistogram2D from the tuple.
   * @param histogram. The histogram to fill.
   * @param evaluatorX. The evaluator object used to compute the "x value"
   *                   to be filled in the histogram.
   * @param evaluatorY. The evaluator object used to compute the "y value"
   *                   to be filled in the histogram.
   *                   Then some : fill(valueX, valueY, 1) is done for each 
   *                   tuple row.
   * @param filter. The filter object used to accept or not the current row.
   *                If rejected, the histogram is not filled.
   */
   virtual bool project(IHistogram2D& histogram,
			const IEvaluator& evaluatorX,
			const IEvaluator& evaluatorY,
			const IFilter& filter) = 0;
   
  /**
   * Fill an IHistogram3D from the tuple.
   */
   virtual bool project(IHistogram3D& histogram,
			const IEvaluator& evaluatorX,
			const IEvaluator& evaluatorY,
			const IEvaluator& evaluatorZ,
			const IFilter& filter) = 0;

};
#endif /* ifndef AIDA_ITUPLE_H */ 

