package hep.aida;
/**
 * The "master" factory from which other factories are obtained.
 * Typically accessed by: 
 * <pre>
 *   IAnalysisFactory af = (IAnalysisFactory) ServiceManager.create("hep.aida.IAnalysisFactory");
 * </pre>
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version $Id: IAnalysisFactory.java,v 1.7 2001/11/23 15:27:48 barrand Exp $
 */
// C++ IAnalysisFactory* af = AIDA_createAnalysisFactory();

/*
extern "C" {
IAnalysisFactory* AIDA_createAnalysisFactory();
}
*/

public interface IAnalysisFactory 
{        
    /**
     * Create an ITreeFactory.
     */
    public ITreeFactory createTreeFactory();
    /**
     * Create an IHistogramFactory.
     * @param tree The ITree which created histograms will be associated
     */
    public IHistogramFactory createHistogramFactory(ITree tree);
    /**
     * Create an ICloudFactory.
     * @param tree The ITree which created clouds will be associated
     */
    public ICloudFactory createCloudFactory(ITree tree);
    /**
     * Create an ITupleFactory.
     * @param tree The ITree which created tuples will be associated
     */
    public ITupleFactory createTupleFactory(ITree tree);
    /**
     * Create an IFunctionFactory.
     * @param tree The ITree which created functions will be associated
     */
    public IFunctionFactory createFunctionFactory (ITree tree);
    /**
     * Create an IEvaluatorFactory.
     */
    public IEvaluatorFactory createEvaluatorFactory ( );
    /**
     * Create an IFilterFactory.
     */
    public IFilterFactory createFilterFactory ( );
    /**
     * Create an IPlotterFactory.
     */
    public IPlotterFactory createPlotterFactory ( );
}
