package hep.aida;

/**
 * Open questions about IAnnotation (GB) :
 *  What is it for ? 
 *  What is the definition for it ?
 *  Do we have, for coherency, to put a annotation() method on all "data classes"
 * (IHistogram, ICloud, IFunction, ITuple) ? 
 *  Do we assume a one to one relationship ?
 *  When storing a "data object" (IHistogram, IFunction, ...)
 * do we have to store/retrieve the associated IAnnotation ?
 * 
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version $Id: IAnnotation.java,v 1.3 2001/11/26 09:08:40 barrand Exp $
 */

public interface IAnnotation {

  /**
   * Add a key/value pair with a given visibility.
   * What visibility is for ?
   * If key alredy exists, override its value ?
   */
  public boolean addItem( String  key, 
			  String  value,
			  boolean visibility);

  /**
   * Remove the item indicated by a given key
   */
  public boolean removeItem( String  key );

  /**
   * Retrieve the value for a given key
   */
  public String  value( String  key);

  /**
   * Retrieve the visibility for a given key
   */
  public boolean visibility( String  key);

  /**
   * Set value for a given key
   */
  public void setValue( String  key, String value);

  /**
   * Set visibility for a given key
   */
  public void setVisibility( String  key, boolean visibility);

  /// For "iterative" access to the objects ...
  /// in future releases we might have an iterator ...

  /// Get the number of items in the Annotation
  public int size();

  /// Individual access to the Annotation-items
  /// This probably should be replaced by a proper iterator
  /// returning a IAnnotationItem object ...
  public String  key(int index);
  public String  value(int index);
  public boolean visibility(int index);

  /// Reset contents
  public void reset();

}
