package hep.aida;

/**
 * User level interface to Histogram.
 *
 * - NOTE: fillable or non-fillable
 *
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version $Id: IHistogram.java,v 1.7 2001/11/29 07:39:40 barrand Exp $
 */
public interface IHistogram extends IConstants {

	/**
	 * Label of the histogram
	 *
	 * @returns null if label was not set.
	 */ 
	public String label();
	
	public void setLabel(String label);
	/**
	 * Title of the histogram (will be set only in the constructor).
	 * @deprecated please use label()
	 */ 
	public String title();    
	/**
         * Get the IAnnotation object.
	 */
    public IAnnotation annotation();
	/**
	 * Returns 1 for one-dimensional histograms, 2 for two-dimensional histograms, and so on.
	 */ 
	public int dimension();    
	/**
	 * Reset contents; as if just constructed.
	 */
	public void reset();    
	/**
	 * Number of in-range entries in the histogram.
	 */ 
	public int entries();    
	/**
	 * Number of all entries in all (both in-range and under/overflow) bins in the histogram.
	 */ 
	public int allEntries();    
	/**
	 * Number of under and overflow entries in the histogram.
	 */ 
	public int extraEntries();    
	/** 
	 * Number of equivalent entries.
	 * @return <tt>SUM[ weight ] ^ 2 / SUM[ weight^2 ]</tt>.
	 */ 
	public double equivalentBinEntries();    
	/**
	 * Sum of in-range bin heights in the histogram.
	 */ 
	public double sumBinHeights();    
    /**
     * Sum of all (both in-range and under/overflow) bin heights in the histogram.
     */ 
    public double sumAllBinHeights();    
	/**
	 * Sum of under/overflow bin heights in the histogram.
	 */ 
	public double sumExtraBinHeights();    
	/**
	 * Minimum height of in-range bins in the histogram.
	 */ 
	public double minBinHeight();    
	/**
	 * Maximum height of in-range bins in the histogram.
	 */ 
	public double maxBinHeight();    

}
