package hep.aida;

/**
 * User level interface to 1D Histogram.
 * 
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version $Id: IHistogram1D.java,v 1.6 2001/11/29 07:40:05 barrand Exp $
 */
public interface IHistogram1D extends IHistogram {
	/**
	 * Fill histogram with weight 1.
	 */ 
	public void fill(double x);    
	/**
	 * Fill histogram with specified weight.
	 */
	public void fill(double x, double weight);    
	/**
	 * Centre of the corresponding bin. As this may not be the simple average 
	 * between upper and lower edge of the bin, it has been moved from IAxis to here.
	 * @param index the bin number (0...N-1) or OVERFLOW or UNDERFLOW.
	 */
        public double binCentre ( int index );
	/**
	 * Number of entries in the corresponding bin (ie the number of times fill was called for this bin).
	 * @param index the bin number (0...N-1) or OVERFLOW or UNDERFLOW.
	 */
	public int binEntries(int index);    
	/**
	 * Total height of the corresponding bin (ie the sum of the weights in this bin).
	 * @param index the bin number (0...N-1) or OVERFLOW or UNDERFLOW.
	 */ 
	public double binHeight(int index);    
	/**
	 * The error on this bin.
	 * @param index the bin number (0...N-1) or OVERFLOW or UNDERFLOW.
	 */ 
	public double binError(int index );    
	/**
	 * Returns the mean of the whole histogram as calculated on filling-time.
	 */ 
	public double mean();    
	/**
	 * Returns the rms of the whole histogram as calculated on filling-time.
	 */ 
	public double rms();    
	/**
	 * Returns the X axis.
	 */
	public IAxis axis();    
	/**
	 * Convenience method, equivalent to <tt>axis().coordToIndex(coord)</tt>.
	 * @see IAxis#coordToIndex(double)
	 */
	public int coordToIndex(double coord);    

    /**
     * Modifies this histogram by adding the contents of h to it.
     *
     * @param h The histogram to be added to this histogram
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
    public void add(IHistogram1D h) throws IllegalArgumentException;
    
}



