package hep.aida;

/**
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version $Id: IHistogramFactory.java,v 1.4 2001/07/24 13:24:04 duns Exp $
 */
public interface IHistogramFactory {

    public IHistogram1D create1D(String nameAndLabel, 
                                 int nBins, 
                                 double lowerEdge, double upperEdge);
                                 
    public IHistogram1D create1D(String name, 
                                 String label,
                                 int nBins, 
                                 double lowerEdge, double upperEdge);
                                 
    public IHistogram1D create1D(String name, 
                                 String label,
                                 int nBins, 
                                 double lowerEdge, double upperEdge,
                                 String options);
                                                         
    public IHistogram1D create1D(String name, 
                                 String label,
                                 double[] binEdge,
                                 String options);
                                                         
                                 
    public IHistogram2D create2D(String nameAndLabel,
                                 int nBinsX, 
                                 double lowerEdgeX, double upperEdgeX,
                                 int nBinsY,
                                 double lowerEdgeY, double upperEdgeY);
                                 
    public IHistogram2D create2D(String name,
                                 String label,
                                 int nBinsX, 
                                 double lowerEdgeX, double upperEdgeX,
                                 int nBinsY,
                                 double lowerEdgeY, double upperEdgeY);
                                 
    public IHistogram2D create2D(String name,
                                 String label,
                                 int nBinsX, 
                                 double lowerEdgeX, double upperEdgeX,
                                 int nBinsY,
                                 double lowerEdgeY, double upperEdgeY,
                                 String options);

    public IHistogram2D create2D(String name,
                                 String label,
                                 double[] binEdgeX, double[] binEdgeY,
                                 String options);

                                
    public IHistogram3D create3D(String nameAndLabel,
                                 int nBinsX, 
                                 double lowerEdgeX, double upperEdgeX,
                                 int nBinsY,
                                 double lowerEdgeY, double upperEdgeY,
                                 int nBinsZ,
                                 double lowerEdgeZ, double upperEdgeZ);
                                 
    public IHistogram3D create3D(String name,
                                 String label,
                                 int nBinsX, 
                                 double lowerEdgeX, double upperEdgeX,
                                 int nBinsY,
                                 double lowerEdgeY, double upperEdgeY,
                                 int nBinsZ,
                                 double lowerEdgeZ, double upperEdgeZ);
                                 
    public IHistogram3D create3D(String name,
                                 String label,
                                 int nBinsX, 
                                 double lowerEdgeX, double upperEdgeX,
                                 int nBinsY,
                                 double lowerEdgeY, double upperEdgeY,
                                 int nBinsZ,
                                 double lowerEdgeZ, double upperEdgeZ,
                                 String options);
                                
    public IHistogram3D create3D(String name,
                                 String label,
                                 double[] binEdgeX, double[] binEdgeY, double[] binEdgeZ,
                                 String options);
                                
    // Methods which create histograms by performing operations on existing histograms
    // 1D Histograms 
    /**
     * Adds two 1D histograms
     *
     * @returns a+b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
    public IHistogram1D add(String nameAndLabel, IHistogram1D a, IHistogram1D b) throws IllegalArgumentException;
    /**
     * Subtracts two 1D histograms
     *
     * @returns a-b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
    public IHistogram1D sub(String nameAndLabel, IHistogram1D a, IHistogram1D b) throws IllegalArgumentException;
    /**
     * Multiplies two 1D histograms
     *
     * @returns a*b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
    public IHistogram1D mul(String nameAndLabel, IHistogram1D a, IHistogram1D b) throws IllegalArgumentException;
    /**
     * Divides two 1D histograms
     *
     * @returns a/b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
    public IHistogram1D div(String nameAndLabel, IHistogram1D a, IHistogram1D b) throws IllegalArgumentException;
    
    // 2D Histograms
    /**
     * Adds two 2D histograms
     *
     * @returns a+b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
    public IHistogram2D add(String nameAndLabel, IHistogram2D a, IHistogram2D b) throws IllegalArgumentException;
    /**
     * Subtracts two 2D histograms
     *
     * @returns a-b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
    public IHistogram2D sub(String nameAndLabel, IHistogram2D a, IHistogram2D b) throws IllegalArgumentException;
    /**
     * Multiplies two 2D histograms
     *
     * @returns a*b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
    public IHistogram2D mul(String nameAndLabel, IHistogram2D a, IHistogram2D b) throws IllegalArgumentException;
    /**
     * Divides two 2D histograms
     *
     * @returns a/b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
    public IHistogram2D div(String nameAndLabel, IHistogram2D a, IHistogram2D b) throws IllegalArgumentException;
    // 3D Histograms
    /**
     * Adds two 3D histograms
     *
     * @returns a+b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
    public IHistogram3D add(String nameAndLabel, IHistogram3D a, IHistogram3D b) throws IllegalArgumentException;
    /**
     * Subtracts two 3D histograms
     *
     * @returns a-b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
    public IHistogram3D sub(String nameAndLabel, IHistogram3D a, IHistogram3D b) throws IllegalArgumentException;
    /**
     * Multiplies two 3D histograms
     *
     * @returns a*b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
    public IHistogram3D mul(String nameAndLabel, IHistogram3D a, IHistogram3D b) throws IllegalArgumentException;
    /**
     * Divides two 3D histograms
     *
     * @returns a/b
     * @throws IllegalArgumentException if histogram binnings are incompatible
     */
    public IHistogram3D div(String nameAndLabel, IHistogram3D a, IHistogram3D b) throws IllegalArgumentException;
      
    // 2D Projections
   /**
	 * Create a projection from a 2D histogram parallel to the X axis.
	 * Equivalent to <tt>sliceX(UNDERFLOW_BIN,OVERFLOW_BIN)</tt>.
	 */
	public IHistogram1D projectionX(String nameAndLabel, IHistogram2D h);
	/**
	 * Create a projection from a 2D histogram parallel to the Y axis.
	 * Equivalent to <tt>sliceY(UNDERFLOW_BIN,OVERFLOW_BIN)</tt>.
	 */
	public IHistogram1D projectionY(String nameAndLabel, IHistogram2D h);
	/**
	 * Slice parallel to the Y axis from a 2D histogram at bin indexY and one bin wide.
	 * Equivalent to <tt>sliceX(indexY,indexY)</tt>.
	 */
	public IHistogram1D sliceX(String nameAndLabel, IHistogram2D h, int indexY);
	/**
	 * Slice parallel to the X axis from a 2D histogram at bin indexX and one bin wide.
	 * Equivalent to <tt>sliceY(indexX,indexX)</tt>.
	 */
	public IHistogram1D sliceY(String nameAndLabel, IHistogram2D h, int indexX);
	/**
	 * Create a slice parallel to the X axis from a 2D histogram, 
	 * between "indexY1" and "indexY2" (inclusive).
	 * The returned IHistogram1D represents an instantaneous snapshot of the
	 * histogram at the time the slice was created.
	 */ 
	public IHistogram1D sliceX(String nameAndLabel, IHistogram2D h, int indexY1, int indexY2);
	/**
	 * Create a slice parallel to the Y axis from a 2D histogram, 
	 * between "indexX1" and "indexX2" (inclusive).
	 * The returned IHistogram1D represents an instantaneous snapshot of the
	 * histogram at the time the slice was created.
	 */ 
	public IHistogram1D sliceY(String nameAndLabel, IHistogram2D h, int indexX1, int indexX2);    
        /**
         * Create a projection parallel to the X axis from a 3D histogram.
         * Equivalent to <tt>sliceXY(UNDERFLOW_BIN,OVERFLOW_BIN)</tt>.
         */
    public IHistogram2D projectionXY(String nameAndLabel, IHistogram3D h);
        /**
         * Create a projection parallel to the Z axis from a 3D histogram.
         * Equivalent to <tt>sliceXZ(UNDERFLOW_BIN,OVERFLOW_BIN)</tt>.
         */
    public IHistogram2D projectionXZ(String nameAndLabel, IHistogram3D h);
        /**
         * Create a projection parallel to the Y axis from a 3D histogram.
         * Equivalent to <tt>sliceYZ(UNDERFLOW_BIN,OVERFLOW_BIN)</tt>.
         */
    public IHistogram2D projectionYZ(String nameAndLabel, IHistogram3D h);
        /**
         * Create a slice perpendicular to the Z axis from a 3D histogram,
         * between "indexZ1" and "indexZ2" (inclusive).
         * The returned IHistogram2D represents an instantaneous snapshot of the
         * histogram at the time the slice was created.
         * The X axis of the returned histogram corresponds to the X axis of this histogram.
         * The Y axis of the returned histogram corresponds to the Y axis of this histogram.
         */
    public IHistogram2D sliceXY(String nameAndLabel, IHistogram3D h, int indexZ1, int indexZ2);
        /**
         * Create a slice perpendicular to the Y axis from a 3D histogram,
         * between "indexY1" and "indexY2" (inclusive).
         * The returned IHistogram2D represents an instantaneous snapshot of the
         * histogram at the time the slice was created.
         * The X axis of the returned histogram corresponds to the X axis of this histogram.
         * The Y axis of the returned histogram corresponds to the Z axis of this histogram.
         */
    public IHistogram2D sliceXZ(String nameAndLabel, IHistogram3D h, int indexY1, int indexY2);
        /**
         * Create a slice perpendicular to the X axis from a 3D histogram,
         * between "indexX1" and "indexX2" (inclusive).
         * The returned IHistogram2D represents an instantaneous snapshot of the
         * histogram at the time the slice was created.
         * The X axis of the returned histogram corresponds to the Y axis of this histogram.
         * The Y axis of the returned histogram corresponds to the Z axis of this histogram.
         */
    public IHistogram2D sliceYZ(String nameAndLabel, IHistogram3D h, int indexX1, int indexX2);
}
