package hep.aida;

import java.io.*;
import java.util.*;

/**
 * User level interface to a Tree.
 *
 * All paths follow unix convention .., ., /, 
 * backslash is the escape character.
 * Relative paths are allowed.
 *
 * NOTE: - this tree keeps a current position (pwd) within the tree.
 * This may be implemented on a per-thread basis.
 *
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version $Id: ITree.java,v 1.4 2001/07/11 19:26:01 tonyj Exp $
 */
public interface ITree {
    
    public IManagedObject find(String path) throws IllegalArgumentException;
    
    public void cd(String path) throws IllegalArgumentException;
    public String pwd();
    
    // All list methods return all managed objects,
    // including folders, but not symlinks
    public Iterator ls();
    public Iterator ls(boolean recursive);
    public Iterator ls(String path) throws IllegalArgumentException;
    public Iterator ls(String path, boolean recursive) throws IllegalArgumentException;
    
    // path = subdir.../dir
    // throws exception if 
    // - subdir does not exist
    // - dir does exist
    // - subdir is not a folder
    public void mkdir(String path) throws IllegalArgumentException;
    
    // path = subdir.../dir
    // throws exception if 
    // - subdir is not a folder
    public void mkdirs(String path) throws IllegalArgumentException; 
    
    public void rmdir(String path);
    // removing a mount point should commit, close and delete the tree object    
    public void rm(String path);
    public String findPath(IManagedObject object) throws IllegalArgumentException;
    public void mv(String oldPath, String newPath) throws IllegalArgumentException;

    public void commit() throws IOException;
    public void cp(String oldPath, String newPath) throws IllegalArgumentException;
    public void cp(String oldPath, String newPath, boolean recursive) throws IllegalArgumentException;
    public void symlink(String path, String alias) throws IllegalArgumentException;
    // a tree can only be mounted once
    /**
     * Mounts a tree within another (target) tree.
     * Example:
     * <pre>
     *     target.mount("/home/tonyj",tree,"/");
     * </pre>
     * @param path The path in the target tree
     * @param tree The tree to mount within the target tree
     * @param treePath the mount point within the tree to be mounted.
     */
    public void mount(String path, ITree tree, String treePath) throws IllegalArgumentException;
    public void unmount(String path) throws IllegalArgumentException;
    // close calls commit before closing
    public void close() throws IOException;
}
