package hep.aida.dev;
import hep.aida.*;



/**
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version 2.0, 06-02-2001
 */

public interface IGristogram extends IConstants
{

	/**
	 * Label of the histogram (will be set only in the constructor).
	 */ 
	public String label();
	/**
	 * to be defined
	 */
    public IAnnotation annotation();
	/**
	 * Returns 1 for one-dimensional histograms, 2 for two-dimensional histograms, and so on.
	 */ 
	public int dimension();    
	/**
	 * Sum of in-range bin heights in the histogram.
	 */ 
	public double sumBinHeights();    
    /**
     * Sum of all (both in-range and under/overflow) bin heights in the histogram.
     */ 
    public double sumAllBinHeights();    
	/**
	 * Sum of under/overflow bin heights in the histogram.
	 */ 
	public double sumExtraBinHeights();    
	/**
	 * Minimum height of in-range bins in the histogram.
	 */ 
	public double minBinHeight();    
	/**
	 * Maximum height of in-range bins in the histogram.
	 */ 
	public double maxBinHeight();        
	/**
	 * Number of entries in the corresponding bin (ie the number of times fill was called for this bin).
	 * @param index the bin number (0...N-1) or OVERFLOW or UNDERFLOW.
	 */
	public double binHeight(int[] index);    
	/**
	 * The error on this bin.
	 * @param index the bin number (0...N-1) or OVERFLOW or UNDERFLOW.
	 */ 
	public double binError(int[] index);    
	/**
	 * Returns the mean of the whole histogram as calculated on filling-time.
	 */ 
	public double mean(int axis);    
	/**
	 * Returns the rms of the whole histogram as calculated on filling-time.
	 */ 
	public double rms(int axis);    
	/**
	 * Returns the axis.
	 */
	public IAxis axis(int axis);    
}
